;-------------------------------------------------------------------
; ForKnox Hidden Data Finder v0.1 (24 February 2004)
; Freeware, Open Source, GPL, Copyleft, whatever you want.
;-------------------------------------------------------------------
;
; It's just a very small utility, quick-and-dirty coded,
; to extract hidden data from whatever file. 
;
; The program version I downloaded from their website and that I
; used for my tests is FortKnox 3.55 with the internal sub-program
; Steganography 2.8. 
;
; I'm not verifying if the file is a real BMP, because the
; main problem of FortKnox is that it treats all files like
; a BMP! Which of course destroys all of the non-BMP ones. Read
; the webpage if you don't get what I mean here.
;
; To use this program: chose which carrier file you want to extract
; data from, and then you will have a new file created called 
; "extracted.bin". 
;
; Remember that you could theorically hide binary data with
; FortKnox, but because of a second bug, if it sees a zero, it will
; stop! And of course binary data obviously contains a lot of 
; zeroes. My program does *not* do like FortKnox: it will *not*
; stop when it meets a zero, and so it will extract everything
; from a file, from first to last byte.
;
; So if you unhide some small text, you will see the extracted 
; text, followed by a lot of random bytes. It's normal. Read my
; webpage about this program to understand why.
;
; Guillermito
; guillermito@pipo.com
; http://www.guillermito2.net
;
; Assemble it with:
;      TASM32 /ml /m3 /z /t FortKnox_Hidden_Text_Finder
;      TLINK32 -Tpe -aa FortKnox_Hidden_Text_Finder,,,import32 
;      use LordPE to realign it, from 4kb to 2 kb.

callW macro   x
extrn x:PROC
call x
endm

   .486
   .model flat

.data

openfilename_struct:
lStructSize       dd openfilename_struct_size
hwndOwner         dd 0
hInstance         dd 0
lpstrFilter       dd offset filter
lpstrCustomFilter dd 0
nMaxCustFilter    dd 0
nFilterIndex      dd 0
lpstrFile         dd offset namebuffer
nMaxFile          dd 255
lpstrFileTitle    dd 0
nMaxFileTitle     dd 32
lpstrInitialDir   dd 0
lpstrTitle        dd 0
Flags             dd 1000h+4h+200000h
nFileOffset       dw 0
nFileExtension    dw 0
lpstrDefExt       dd 0
lCustData         dd 0
lpfnHook          dd 0
lpTemplateName    dd 0
openfilename_struct_size equ $-offset openfilename_struct

filter            db "all files *.*",0,"*.*",0,0
namebuffer        db 255 dup(0)
result_title      db "FortKnox Hidden Text Finder by Guillermito", 0
done              db "Done. Check extracted.bin.",0

file_name             db "extracted.bin",0
file_handle2          dd ?
file_size             dd ?
file_handle           dd ?
file_mem_buffer       dd ?
file_nb_bytes_read    dd ?
pixels_data_start     dd ?
size_hidden_data      dd ?
buffer_for_extraction dd ?
buffer_end            dd ?

.code

programme:

;-------- choose a file -------------

push offset openfilename_struct
callW GetOpenFileNameA             ;choose file
test eax, eax
jz end

;-------- open it --------------

push 0
push 80h                          ;FILE_ATTRIBUTE_NORMAL
push 3                            ;OPEN_EXISTING
push 0
push 0
push 80000000h+40000000h          ;GENERIC_READ+GENERIC_WRITE
push dword ptr [lpstrFile]
callW CreateFileA                 ;open file
inc eax
jz end
dec eax
mov file_handle, eax

;---------- get its size ---------

push 0
push dword ptr file_handle
callW GetFileSize                 ;get file size in bytes
inc eax
jz close_file
dec eax
mov file_size, eax

;--------- alloc memory for file --------

mov eax, file_size
shl eax, 2
add eax, 1024
push eax                         ;number of bytes to allocate
push 40h                         ;40h=LMEM_ZEROINIT
callW LocalAlloc
test eax, eax
jz close_file
mov file_mem_buffer, eax

;------- read entire file ------------

push 0
push offset file_nb_bytes_read
push file_size
push file_mem_buffer
push file_handle
callW ReadFile
test eax, eax
jz free_memory
mov eax, file_nb_bytes_read
cmp eax, file_size
jnz free_memory

;------ Prepare the extraction -----------

mov edi, file_mem_buffer
add edi, file_size
mov buffer_for_extraction, edi

mov esi, file_mem_buffer
add esi, 55

mov ecx, file_size
sub ecx, 55
shr ecx, 3

;------------- extract everything --------------

extract_everything:
push ecx
mov ecx, 6
xor ebx, ebx
xor eax, eax

extract_a_byte:
lodsb				;get byte
and al, 3			;eliminate everything except 2 LSBs
rol al, cl			;put the 2 bits where they belong in the byte
or bl, al			;along with their friends
dec ecx
dec ecx
cmp ecx, -2
jne extract_a_byte
xchg eax, ebx
stosb

pop ecx
loop extract_everything
    
;----- calculate how long we have -----

mov buffer_end, edi
sub edi, buffer_for_extraction
mov size_hidden_data, edi

;-------- display MessageBox -----------

push 0
push offset result_title
push offset done
push 0
callW MessageBoxA
jmp save_file

;------- open a new file on the disk ----------------

save_file:
push 0                  ;handle of file with attributes to copy
push 80h                ;file attributes (80h=FILE_ATTRIBUTE_NORMAL)
push 2                  ;how to create (2=OPEN_ALWAYS)
push 0                  ;address of security descriptor
push 0                  ;share mode (0=Prevents the file from being shared)
push 40000000h          ;access (read-write) mode (40000000h=GENERIC_WRITE)
push offset file_name   ;address of name of the file
call CreateFileA
mov file_handle2, eax

;------- write buffer in it ----------------

push 0
push offset file_nb_bytes_read  ;address of number of bytes written 
push size_hidden_data           ;number of bytes to write
push buffer_for_extraction      ;address of data to write to file
push file_handle2               ;handle of file to write to
callW WriteFile

;------- close it ----------------

push file_handle2
callW CloseHandle
jmp free_memory

;------- close memory ----------------

free_memory:
push file_mem_buffer
callW LocalFree

;------- close file ----------------

close_file:
push dword ptr file_handle
callW CloseHandle

;----------- exit ----------

end:
push -1
callW ExitProcess

end programme
